@@ -1,6 +1,6 @@
 package Mojo::IRC::Server;
 use strict;
-$Mojo::IRC::Server::VERSION = "1.7.1";
+$Mojo::IRC::Server::VERSION = "1.7.2";
 use Encode;
 use Encode::Locale;
 use Carp;
@@ -95,6 +95,7 @@ sub new_user{
         elsif($msg->{command} eq "WHOIS"){$user->emit(whois=>$msg);$s->emit(whois=>$user,$msg);}
         elsif($msg->{command} eq "LIST"){$user->emit(list=>$msg);$s->emit(list=>$user,$msg);}
         elsif($msg->{command} eq "TOPIC"){$user->emit(topic=>$msg);$s->emit(topic=>$user,$msg);}
+        else{$user->send($user->serverident,"421",$user->nick,$msg->{command},"Unknown command");}
     });
 
     $user->io->on(error=>sub{
@@ -108,6 +109,7 @@ sub new_user{
         $s->emit(close_user=>$user);
         $user->emit("close");
     });
+    $user->on(pass=>sub{my($user,$msg) = @_;my $pass = $msg->{params}[0]; $user->pass($pass);});
     $user->on(nick=>sub{my($user,$msg) = @_;my $nick = $msg->{params}[0];$user->set_nick($nick)});
     $user->on(user=>sub{my($user,$msg) = @_;
         if(defined $user->search_user(user=>$msg->{params}[0])){
@@ -193,7 +195,10 @@ sub new_user{
             if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
             if(defined $channel_mode and $channel_mode eq "b"){
                 $user->send($user->serverident,"368",$user->nick,$channel_name,"End of channel ban list");
-            }   
+            }
+            elsif(defined $channel_mode and $channel_mode ne "b") {
+                $channel->set_mode($user,$channel_mode);
+            }
             else{
                 $user->send($user->serverident,"324",$user->nick,$channel_name,'+'.$channel->mode);
                 $user->send($user->serverident,"329",$user->nick,$channel_name,$channel->ctime);
@@ -203,15 +208,36 @@ sub new_user{
             my $nick = $msg->{params}[0];
             my $mode = $msg->{params}[1];
             if(defined $mode){$user->set_mode($mode)}
-            else{$user->send($user->ident,"MODE",$user->nick,'+'.$user->mode)}
+            else{$user->send($user->serverident,"221",$user->nick,'+'.$user->mode)}
         }    
     });
     $user->on(who=>sub{my($user,$msg) = @_;
-        my $channel_name = $msg->{params}[0];
-        my $channel = $user->search_channel(name=>$channel_name);
-        if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
-        $user->send($user->serverident,"352",$user->nick,$channel_name,$user->user,$user->host,$user->servername,$user->nick,"H","0 " . $user->realname);
-        $user->send($user->serverident,"315",$user->nick,$channel_name,"End of WHO list");
+        if(substr($msg->{params}[0],0,1) eq "#" ){
+            my $channel_name = $msg->{params}[0];
+            my $channel = $user->search_channel(name=>$channel_name);
+            if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
+            for($channel->users){
+                $user->send($user->serverident,"352",$user->nick,$channel_name,$_->user,$_->host,$_->servername,$_->nick,"H","0 " . $_->realname);
+            }
+            $user->send($user->serverident,"315",$user->nick,$channel_name,"End of WHO list");
+        }
+        else{
+            my $nick = $msg->{params}[0];
+            my $u = $user->search_user(nick=>$nick);
+            if(defined $u){
+                my $channel_name = "*";
+                if($u->is_join_channel()){
+                    my $last_channel = ($u->channels)[-1];
+                    $channel_name = $last_channel->name;
+                }
+                $user->send($user->serverident,"352",$user->nick,$channel_name,$u->user,$u->host,$u->servername,$u->nick,"H","0 " . $u->realname);
+                $user->send($user->serverident,"315",$user->nick,$nick,"End of WHO list");
+            }
+            else{
+                $user->send($user->serverident,"401",$user->nick,$nick,"No such nick");
+            }
+            
+        }
     });
     $user->on(whois=>sub{my($user,$msg) = @_;});
     $user->on(list=>sub{my($user,$msg) = @_;
@@ -223,10 +249,15 @@ sub new_user{
     });
     $user->on(topic=>sub{my($user,$msg) = @_;
         my $channel_name = $msg->{params}[0];
-        my $topic = $msg->{params}[1];
         my $channel = $user->search_channel(name=>$channel_name);
-        if(not defined $channel){$user->send($user->serverident,"403",$channel_name,"No such channel");return}
-        $channel->set_topic($user,$topic);
+        if(not defined $channel){$user->send($user->serverident,"403",$user->nick,$channel_name,"No such channel");return}
+        if(defined $msg->{params}[1]){
+            my $topic = $msg->{params}[1];
+            $channel->set_topic($user,$topic);
+        }
+        else{
+            $user->send($user->serverident,"332",$user->nick,$channel_name,$channel->topic);
+        }
     });
     $user;
 }
@@ -257,6 +288,8 @@ sub add_user{
         my $nick = $user->nick;
         $nick =~s/\s|\@|!//g;$nick = '未知昵称' if not $nick;
         $user->nick($nick);
+        my $u = $s->search_user(nick=>$user->nick,virtual=>1,id=>$user->id);
+        return $u if defined $u;
         while(1){
             my $u = $s->search_user(nick=>$user->nick);
             if(defined $u){
