@@ -3,6 +3,7 @@ use Mojo::IRC::Server::Base 'Mojo::IRC::Server::Object';
 use List::Util qw(first);
 has [qw(id name io)];
 has user    => '*';
+has pass    => undef;
 has nick    => '*';
 has mode    => 'i';
 has buffer  => '';
@@ -36,7 +37,6 @@ sub set_nick{
         if($user->is_virtual){
             $s->{_server}->remove_user($user);
             $s->once(close=>sub{$s->{_server}->add_user($user)});
-            $s->send($s->ident,NICK => $nick);
             $s->broadcast($s->ident,NICK => $nick);
             $s->info("[" . $s->nick . "] 修改昵称为 [$nick]");
             $s->nick($nick);
@@ -48,7 +48,6 @@ sub set_nick{
         }
     }
     else{
-        $s->send($s->ident,NICK => $nick);
         $s->broadcast($s->ident,NICK => $nick);
         $s->info("[" . $s->nick . "] 修改昵称为 [$nick]");
         $s->nick($nick);
@@ -78,14 +77,17 @@ sub join_channel{
     my $channel;
     $channel = ref($_[0]) eq "Mojo::IRC::Server::Channel"?$_[0]:$s->search_channel(id=>$_[0]);
     return if not defined $channel;
-    push @{$s->channel},$channel->id if not $s->is_join_channel($channel->id);
-    $channel->add_user($s->id);
-    $channel->broadcast($s->ident,"JOIN",$channel->name);
+    if(not $s->is_join_channel($channel->id)){
+        push @{$s->channel},$channel->id;
+        $channel->add_user($s->id);
+        $channel->broadcast($s->ident,"JOIN",$channel->name);
+    }
+    else{$s->send($s->ident,"JOIN",$channel->name);} 
     $s->send($s->serverident,"332",$s->nick,$channel->name,$channel->topic);
     $s->send($s->serverident,"353",$s->nick,'=',$channel->name,join " ",map {$_->nick} $channel->users);
     $s->send($s->serverident,"366",$s->nick,$channel->name,"End of NAMES list");
     #$s->send($s->serverident,"329",$s->nick,$channel->name,$channel->ctime);
-    $s->info("[" . $s->name . "] 加入频道 " . $channel->name);    
+    $s->info("[" . $s->name . "] 加入频道 " . $channel->name);
 }
 sub part_channel{
     my $s = shift;
